from .format import (
    Conversation,
    ConverstationAudio,
    ConverstationImage,
    ConverstationText,
    DatasetOutput,
)
from torch.utils.data import Dataset
from datasets import load_dataset


class GigaspeechDataset(Dataset):
    def __init__(self, audio_processor=None, text_processor=None, split="train"):
        """
        vis_root (string): Root directory of images (e.g. coco/images/)
        ann_root (string): directory to store the annotation file
        """

        self.audio_processor = audio_processor
        self.text_processor = text_processor
        gs = load_dataset("speechcolab/gigaspeech", "xs")
        self.data = gs[split]

    def __len__(self):
        return len(self.data)

    def __getitem__(self, index):
        sample = self.data[index]
        audio = sample["audio"]["array"]
        sampling_rate = sample["audio"]["sampling_rate"]
        text = sample["text"]

        if self.audio_processor is not None:
            audio = self.audio_processor(audio)
        if self.text_processor is not None:
            text = self.text_processor(text)

        chat = [
            Conversation(
                role="user",
                content=[
                    ConverstationAudio(type="audio", audio_url=""),
                    ConverstationText(
                        type="text", text="Please convert the audio to text"
                    ),
                ],
            ),
            Conversation(
                role="assistant", content=[ConverstationText(type="text", text=text)]
            ),
        ]

        return DatasetOutput(
            audios=[(audio, sampling_rate)],
            chat=chat,
            original=sample,
        )


class GigaspeechDatasetForGeneration(GigaspeechDataset):

    def __getitem__(self, index):
        sample = self.data[index]
        audio = sample["audio"]["array"]
        sampling_rate = sample["audio"]["sampling_rate"]
        text = sample["text"]

        if self.audio_processor is not None:
            audio = self.audio_processor(audio)
        if self.text_processor is not None:
            text = self.text_processor(text)

        chat = [
            Conversation(
                role="user",
                content=[
                    ConverstationAudio(type="audio", audio_url=""),
                    ConverstationText(
                        type="text", text="Please convert the audio to text"
                    ),
                ],
            ),
        ]

        return DatasetOutput(
            audios=[(audio, sampling_rate)],
            chat=chat,
            answer=text,
            original=sample,
        )


def test_gigaspeech():
    dataset = GigaspeechDataset(
        split="train",
    )
    print(dataset[0])
    assert len(dataset) > 0
    assert len(dataset[0]["chat"]) > 0
    dataset = GigaspeechDatasetForGeneration(
        split="train",
    )
    print(dataset[0])
    assert len(dataset) > 0
    assert len(dataset[0]["chat"]) > 0
